/**************************************************************************
* libXicq - Originally written by tnc (xtrophy@it.dk)                     *
*'''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''*
* contact.cpp                                                             *
* Handles the contact list                                                *
**************************************************************************/


#include "contact.h"


CONTACT contactList[MAXCONTACT];
int  contactNum;


/*
** 
** Reads the contact list from a file
** Returns 0 of the contact file was found and -1 if not
*/
int contact_init(char *contactPath)
{
    ifstream contactFile(contactPath, ios::in);
    if (!contactFile)
    {
        char tmp[BUFSIZE];
        sprintf(tmp, "Couldn't open %s.", contactPath);
        misc_log(tmp);

        contactNum = 0;

        return -1;
    }
    else
    {
        contactNum = -1;
        char string[BUFSIZE];
        char *string2;
        char *tmp;
        while (contactFile.getline(string, BUFSIZE))
        {
            contactNum++;

            contactList[contactNum].status = STATUS_OFFLINE;
            
            string2 = string;
            //string2 = strdup(string);
            
            tmp = strsep(&string2, ":");
            contactList[contactNum].uin = atoi(tmp);

            tmp = strsep(&string2, ":");
            strcpy(contactList[contactNum].nick, tmp);

            tmp = strsep(&string2, ":");
            if (!tmp)
                contactList[contactNum].ip = inet_addr("0.0.0.0");

            tmp = strsep(&string2, "");
            if (!tmp)
                contactList[contactNum].real_ip = inet_addr("0.0.0.0");

            contactList[contactNum].port = 0;

            
            if (contactList[contactNum].uin == 0)
                contactNum--;

            free(string2);
        }

        return 0;
    }
}


/*
** 
** Return the number of entries in the contactList
*/
int contact_length()
{
    return contactNum + 1;
}


/*
** 
** Returns an entry in the contact list, or an entry with uin 0 on
** error
*/
CONTACT contact_returnItemNum(int num)
{
    if (num <= contactNum)
        return contactList[num];
    else
    {
        CONTACT contact;
        contact.uin = 0;
        
        return contact;
    }
}


/*
** 
** Convert uin to nick
*/
char *contact_uin2nick(unsigned long int uin)
{
    for (int i = 0; i <= contactNum; i++)
    {
        if (uin == contactList[i].uin)
            return contactList[i].nick;
    }
    //char tmp[BUFSIZE];
    char *tmp;
    tmp = (char *) malloc(BUFSIZE);
    sprintf(tmp, "%ld", uin);
    return tmp;
}


/*
** 
** Convert nick to uin
*/
unsigned long int contact_nick2uin(char *nick)
{
    for (int i = 0; i <= contactNum; i++)
    {
        if (!strcasecmp(nick, contactList[i].nick))
            return contactList[i].uin;
    }

    return(atoi(nick));
}


/*
** 
** Return entry which match uin, or entry with uin 0 on error
*/
CONTACT contact_returnContact(unsigned long int uin)
{
    for (int i = 0; i <= contactNum; i++)
    {
        if (uin == contactList[i].uin)
        {
            return contactList[i];
        }
    }

    CONTACT contact;
    contact.uin = 0;
    return contact;
}


/*
** 
** Update an entry
** Return 0 on success, -1 on error
*/
int contact_updateContact(CONTACT update_contact)
{
    for (int i = 0; i <= contactNum; i++)
    {
        if (update_contact.uin == contactList[i].uin)
        {
            contactList[i] = update_contact;
            return 0;
        }
    }

    return -1;
}


char *contact_status2str(long int status) {
    switch (status) {
        case 0x0:
        // This MIGHT be DND.. 
        case 0x10013:
        case 0x10000:
        // This MIGHT be N/A..
        case 0x10005:
        case 0x20000:
        case 0x30000: {
            return "Available";
            break;
	}
        case 0x1:
        case 0x10001:
        case 0x20001:
        case 0x30001:
            return "Away";
            break;
        case 0x5:
        case 0x20005:
        case 0x30005:
            return "N/A";
            break;
        case 0x11:
        case 0x20011:
        case 0x30011:
            return "Occupied";
            break;
        case 0x13:
	case 0x20013:
        case 0x30013:
            return "DND";
            break;
        case 0x20:
        case 0x20020:
        case 0x30020:
            return "Free for chat";
            break;
        case 0x100:
            return "Invisible";
            break;
        case 0xFFFFFFFF:
            return "Offline";
        default:
            //char string[BUFSIZE];
	    char *string;
	    string = (char *) malloc(BUFSIZE);
            sprintf(string, "Unknown(%ld)", status);
            return string;
            break;
    }

    return '\0';
}// contact_status2str






