/**************************************************************************
* libXicq - Originally written by tnc (xtrophy@it.dk)                     *
*'''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''*
* net.cpp                                                                 *
* network functions                                                       *
**************************************************************************/


#include "net.h"

#define MAXWATCH 10

extern LIB_INFORMATION lib_information;
extern NET_INFORMATION net_information;


struct  sockaddr_in     client,         // Client and server address info
                        server;         
int                     recvd,          // Number of bytes recieved
                        sent;           // Number of bytes sent
int                     structlength;   // Length of sockaddr structure

unsigned short  int     lastRecvd[MAXWATCH]; // Last received sequences


/*
**
** Initializes network functions - Called from icq_init()
** Returns -1 if something goes wrong
*/
int net_init(USER_CONFIGURATION *user_cfg)
{

    net_information.serverAddr = user_cfg->serverAddr;
    net_information.serverPort = user_cfg->serverPort;
    net_information.sock       = net_udpGetSocket();
    net_information.clientAddr = strdup("127.0.0.1");
    net_information.clientPort = user_cfg->clientPort;

    for (int i = 0; i < MAXWATCH; i++)
	lastRecvd[i] = 0;
    
    if (DEBUG)
    {
        char string[BUFSIZE];
        if (net_information.sock != -1)
            sprintf(string, "Network initialized - Got socket %d.", net_information.sock);
        else
            sprintf(string, "Could not initialize network.");
        misc_log(string);
    }

    if (net_information.sock != -1)
        return 0;
    else
        return -1;
}



/*
**
** Finds a socket we can use for UDP transfers
** Returns the socket, or -1 if we didn't get one.
*/
int net_udpGetSocket()
{
    net_information.sock = -1;
    
    struct in_addr *addr;               // Internet address of server
    int sock;                           // Our socket that we will talk through

    struct hostent *host;
    static struct in_addr saddr;
    saddr.s_addr = inet_addr(net_information.serverAddr);
    if (saddr.s_addr != (unsigned long int)-1)
        addr = &saddr;
    else
    {
        host = gethostbyname(net_information.serverAddr);
        if (host != NULL)
            addr = (struct in_addr *) *host->h_addr_list;
        else
        {
            perror("libXicq: lookup");
            return -1;
        }
    }

    sock = socket(AF_INET, SOCK_DGRAM, 0);
    if (sock < 0)
    {
        perror("libXicq: socket");
        return -1;
    }

    /* Set up the server info */
    memset((char *) &server, 0, sizeof(server));
    server.sin_family = AF_INET;
    server.sin_addr.s_addr = addr->s_addr;
    server.sin_port = htons(net_information.serverPort);//port;

    /* Set up another structure for our local socket */
    memset((char *) &client, 0, sizeof(client));
    client.sin_family = AF_INET;
    client.sin_addr.s_addr = htonl(INADDR_ANY);
    client.sin_port = htons(0);
    if (bind(sock, (struct sockaddr *) &client, sizeof(client)) < 0)
    {
        perror("libXicq: bind");
        return -1;
    }

    return sock;
}



/*
**
** Sends an UDP packet to the server
** Returns the number of written bytes, or -1 if something goes wrong
*/
int net_udpSend(unsigned char *mesg, int length)
{
    sent = sendto(net_information.sock, mesg, length, 0,
                  (struct sockaddr *) &server, sizeof(server));
    if (sent < length)
    {
        perror("libXicq: sendto");
        return -1;
    }

    return sent;
}



/*
**
** Receives an UDP packet from the server
** Returns the number of read bytes, or -1 if something goes wrong
*/
int net_udpRecv(unsigned char *mesg, int length)
{
    structlength = sizeof(client);
    recvd = recvfrom(net_information.sock, mesg, length, 0,
                     (struct sockaddr *) &client, &structlength);
    if (recvd < 0)
    {
        perror("libXicq: recvfrom");
        return -1;
    }

    // Remove dublicate packets - not acks - (Our ack got lost)
    unsigned short int *cmd = (unsigned short int *)(mesg + 2);
    if (*cmd != S_ACK)
    {
	unsigned short int *seq = (unsigned short int *)(mesg + 4);
	for (int i = 0; i < MAXWATCH; i++)
	    if (*seq == lastRecvd[i] && *seq != 0)
	    {
		if (DEBUG)
		    misc_log("Got duplicate packet, acking.");
		send_ack(*seq);
		return 0;
	    }

	for (int i = 0; i < MAXWATCH - 1; i++)
	    lastRecvd[i] = lastRecvd[i + 1];
	lastRecvd[MAXWATCH - 1] = *seq;
    }

    return recvd;

}



