/**************************************************************************
* libXicq - Originally written by tnc (xtrophy@it.dk)                     *
*'''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''*
* packets.cpp                                                             *
* Create packets                                                          *
**************************************************************************/


#include "packets.h"


extern NET_INFORMATION net_information;
extern USER_CONFIGURATION user_config;

ICQ_PACKET packet;

int c_sequence = 1;         // Sequence number
int s_sequence = 1;         // Search sequence


/*
** 
** Creates a header
*/
void packets_header(ICQ_PACKET &packet)
{
    unsigned short int version  = (unsigned short int)ICQ_VER;
    unsigned short int command  = (unsigned short int)packet.command;
    unsigned short int sequence = (unsigned short int)packet.sequence;
    unsigned long  int uin      =  (unsigned long int)user_config.uin;

    unsigned char *tmp;
    tmp = packet.data;

    memcpy(tmp, &version, 2);   tmp += 2;
    memcpy(tmp, &command, 2);   tmp += 2;
    memcpy(tmp, &sequence, 2);  tmp += 2;
    memcpy(tmp, &uin, 4);       tmp += 4;

    packet.length = tmp - packet.data;
}


/*
** 
** Creates an ack packet
*/
ICQ_PACKET packets_ack(int sequence)
{
    packet.command = C_ACK;
    packet.sequence = sequence;
    packets_header(packet);

    return packet;
}


/*
** 
** Creates a message ack packet
*/
ICQ_PACKET packets_msgAck(int sequence)
{
    packet.command = C_ACK_MSG;
    packet.sequence = sequence;
    packets_header(packet);

    return packet;
}


/*
** 
** Creates a keep alive packet
*/
ICQ_PACKET packets_keepAlive()
{
    packet.command = C_KEEP_ALIVE;
    packet.sequence = c_sequence++;
    packets_header(packet);

    return packet;
}


/*
** 
** Creates a login packet
*/
ICQ_PACKET packets_login()
{
    packet.command = C_LOGIN;
    packet.sequence = c_sequence++;
    packets_header(packet);
    
    unsigned long  int port     = net_information.clientPort;
    unsigned short int pwdlen   = strlen(user_config.password) + 1;
    char pwd[NAMESIZE];
    strcpy(pwd, user_config.password);
    unsigned long int	X1	= 120;
    unsigned long int	ip	= inet_addr(net_information.clientAddr);
    unsigned char      	X2	= 04;
    unsigned long int	status	= 0;
    unsigned long int	X3	= 2;
    unsigned short int	lseq	= 1;
    unsigned long int	X4	= 0;
    unsigned long int	X5	= 7864328;

    unsigned char *tmp;
    tmp = packet.data + packet.length;

    memcpy(tmp, &port, 4);      tmp += 4;
    memcpy(tmp, &pwdlen, 2);    tmp += 2;
    memcpy(tmp, pwd, pwdlen);   tmp += pwdlen;
    memcpy(tmp, &X1, 4);        tmp += 4;
    memcpy(tmp, &ip, 4);     	tmp += 4;
    memcpy(tmp, &X2, 1);      	tmp += 1;
    memcpy(tmp, &status, 4);    tmp += 4;
    memcpy(tmp, &X3, 4);       	tmp += 4;
    memcpy(tmp, &lseq, 2);     	tmp += 2;
    memcpy(tmp, &X4, 4);       	tmp += 4;
    memcpy(tmp, &X5, 4);       	tmp += 4;

    packet.length = tmp - packet.data;

    return packet;
}


/*
** 
** This is sent during login, don't know the exact purpose
*/
ICQ_PACKET packets_login_1()
{
    packet.command = C_LOGIN_1;
    packet.sequence = c_sequence++;
    packets_header(packet);

    return packet;
}


/*
** 
** Creates a contactList
*/
ICQ_PACKET packets_contactList()
{
    packet.command = C_CONTACT_LIST;
    packet.sequence = c_sequence++;
    packets_header(packet);

    

    unsigned char *tmp;
    tmp = packet.data + packet.length;

    unsigned short int length = contact_length();
    memcpy(tmp, &length, 1);    tmp += 1;

    CONTACT contact;
    for (int i = 0; i < contact_length(); i++)
    {
        contact = contact_returnItemNum(i);
        memcpy(tmp, &contact.uin, 4);   tmp += 4;
    }

    packet.length = tmp - packet.data;

    return packet;
}


/*
** 
** Creates a message to be sent through the server
*/
ICQ_PACKET packets_messageS(unsigned long int receiver, char *message)
{
    packet.command = C_SEND_MESSAGE;
    packet.sequence = c_sequence++;
    packets_header(packet);

    unsigned char *tmp;
    tmp = packet.data + packet.length;

    unsigned short int type = 1; // Normal message
    unsigned short int msgLength = strlen(message) + 1;

    memcpy(tmp, &receiver, 4);          tmp += 4;
    memcpy(tmp, &type, 2);              tmp += 2;
    memcpy(tmp, &msgLength, 2);         tmp += 2;
    memcpy(tmp, message, msgLength);    tmp += msgLength;

    packet.length = tmp - packet.data;
    
    return packet;
}


/*
**
** Creates a URL to be sent through the server.
*/
ICQ_PACKET packets_urlS(unsigned long int receiver, char *url, char *desc)
{
    packet.command = C_SEND_MESSAGE;
    packet.sequence = c_sequence++;
    packets_header(packet);

    unsigned char *tmp;
    tmp = packet.data + packet.length;

    unsigned short int type = 4; // URL

    unsigned char message[BUFSIZE];
    bzero(message, BUFSIZE);
    strcpy((char *)message, desc);
    message[strlen((char *)message)] = 0xFE;
    message[strlen((char *)message) + 1] = '\0';


    strcat((char *)message, url);

    unsigned short int msgLength = strlen((char *)message) + 1;

    memcpy(tmp, &receiver, 4);          tmp += 4;
    memcpy(tmp, &type, 2);              tmp += 2;
    memcpy(tmp, &msgLength, 2);         tmp += 2;
    memcpy(tmp, message, msgLength);    tmp += msgLength;

    packet.length = tmp - packet.data;

    return packet;
}

  
/*
** 
** Change your online status
*/
ICQ_PACKET packets_statusChange(unsigned long int status)
{
    packet.command = C_STATUS_CHANGE;
    packet.sequence = c_sequence++;
    packets_header(packet);

    unsigned char *tmp;
    tmp = packet.data + packet.length;

    memcpy(tmp, &status, 4);            tmp += 4;

    packet.length = tmp - packet.data;

    return packet;
}


/*
** 
** Search for info on a user (UIN search)
*/
ICQ_PACKET packets_uinSearch(unsigned long int uin)
{
    packet.command = C_SEARCH_UIN;
    packet.sequence = c_sequence++;
    packets_header(packet);

    unsigned char *tmp;
    tmp = packet.data + packet.length;

    memcpy(tmp, &s_sequence, 2);   tmp += 2;
    memcpy(tmp, &uin, 4);               tmp += 4;

    packet.length = tmp - packet.data;

    s_sequence++;

    return packet;
}


/*
** 
** Send a text code to the server. Used for logoff etc
*/
ICQ_PACKET packets_sendTextCode(char *textCode, unsigned short int X1)
{
    packet.command = C_SEND_TEXT_CODE;
    packet.sequence = c_sequence++;
    packets_header(packet);

    unsigned char *tmp;
    tmp = packet.data + packet.length;

    unsigned short int  length = strlen(textCode) + 1;

    memcpy(tmp, &length, 2);            tmp += 2;
    memcpy(tmp, &textCode, length);     tmp += length;
    memcpy(tmp, &X1, 2);                tmp += 2;

    packet.length = tmp - packet.data;
    
    return packet;
}


