/**************************************************************************
* libXicq - Originally written by tnc (xtrophy@it.dk)                     *
*'''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''*
* parse.cpp                                                               *
* Parse incomming packets                                                 *
**************************************************************************/


#include "parse.h"


extern USER_CONFIGURATION user_config;
extern CALLBACK_FUNCTIONS callback;


/*
** Parse received packets and activate the callback function for it
** Returns -1 if something goes wrong
*/
int parse(ICQ_PACKET *packet)
{
    switch (packet->command)
    {
        case S_ACK:// Server acked a received package
            {
                if (DEBUG)
                    misc_log("Got ack.");

                break;
            }
        case S_LOGIN_REPLY:// Server acked out login packet
            {
                if (DEBUG)
                    misc_log("Got login reply.");

                send_ack(packet->sequence);
                user_config.status = user_config.defaultStatus;
                send_statusChange(user_config.status);

                send_login_1();
                send_contactList();

                (*callback.loggedIn)();

                break;
            }
        case S_USER_ONLINE:// Someone from the contact list is online
            {
                if (DEBUG)
                    misc_log("Got user online.");
                
                parse_S_USER_ONLINE(packet);
                break;
            }
        case S_USER_OFFLINE:// Someone logged out
            {
                if (DEBUG)
                    misc_log("Got user offline.");
                
                parse_S_USER_OFFLINE(packet);
                break;
            }
        case S_STATUS_UPDATE:// A user have changed status
            {
                if (DEBUG)
                    misc_log("Got user status change.");
                
                parse_S_STATUS_UPDATE(packet);
                break;
            }
        case S_GET_MESSAGE:// MSG sent while online
            {
                send_ack(packet->sequence);
                parse_S_GET_MESSAGE(packet, 1);
                break;
            }
        case S_RECEIVE_MESSAGE:// MSG send while offline
            {
                send_ack(packet->sequence);
                send_msgAck(packet->sequence);
                parse_S_GET_MESSAGE(packet, 0);
                break;
            }
	case S_USER_FOUND: // User matched our query
	    {
		send_ack(packet->sequence);
		parse_S_USER_FOUND(packet);
		break;
	    }
	case S_END_OF_SEARCH:// No more users matching, or too many found
	    {
		send_ack(packet->sequence);
		break;
	    }
        case S_GO_AWAY:// We've been idle,
        case S_DISCONNECTED:// We got disconnected?
            {
                contact_init(user_config.contactList);
                (*callback.disconnected)();
            }
        case S_REPLY_X1:// Unknown packet, sent during login
            {
                if (DEBUG)
                    misc_log("Got unknown login reply.");
                
                send_ack(packet->sequence);
                break;
            }
        case S_REPLY_X2:// Unknown packet, sent during login
            {
                if (DEBUG)
                    misc_log("Got unknown login reply.");
                
                send_ack(packet->sequence);
                break;
            }
        default:
            {
                //if (DEBUG)
                {
                    char tmp[BUFSIZE];
                    sprintf(tmp, "Unknown packet, CMD: %04X - SEQ: %d", packet->command, packet->sequence);
                    misc_log(tmp);
                }

                send_ack(packet->sequence);
            }
    }

    //misc_printPacket(packet);
    return -1;
}


/*
** 
** User from contact list is online, find out who, and notify the UI
*/
void parse_S_USER_ONLINE(ICQ_PACKET *packet)
{
    CONTACT contact;

    unsigned long int *uin;
    unsigned long int *status;
    uin    = (unsigned long int *)(packet->data + 6);
    status = (unsigned long int *)(packet->data + 23);

    contact = contact_returnContact(*uin);

    contact.uin    = *uin;
    contact.status = *status;

    contact_updateContact(contact);

    send_ack(packet->sequence);

    (*callback.updateContact)(*uin);
}


/*
** 
** User from contact list is offline, find out who, and notify the UI
*/
void parse_S_USER_OFFLINE(ICQ_PACKET *packet)
{
    CONTACT contact;
    unsigned long int *uin;
    uin = (unsigned long int *)(packet->data + 6);

    contact = contact_returnContact(*uin);

    contact.uin    = *uin;
    contact.status = STATUS_OFFLINE;

    contact_updateContact(contact);

    send_ack(packet->sequence);

    (*callback.updateContact)(*uin);
}


/*
** 
** User from contact list has changed status, find out who, and notify the UI
*/
void parse_S_STATUS_UPDATE(ICQ_PACKET *packet)
{
    CONTACT contact;
    unsigned long int *uin;
    unsigned long int *status;
    uin    = (unsigned long int *)(packet->data + 6);
    status = (unsigned long int *)(packet->data + 10);

    contact = contact_returnContact(*uin);

    contact.uin    = *uin;
    contact.status = *status;

    contact_updateContact(contact);

    send_ack(packet->sequence);

    (*callback.updateContact)(*uin);
}


/*
** 
** Got a message packet, find out if it contains a message or an URL
*/
void parse_S_GET_MESSAGE(ICQ_PACKET *packet, int online)
{
    unsigned short int *type;
    
    if (online)
         type = (unsigned short int *)(packet->data + 10);
    else
        type = (unsigned short int *)(packet->data + 16);

    switch (*type)
    {
        case 0x0001:// Message
            {
                if (DEBUG)
                    misc_log("Got message.");
                
                parse_message(packet, online);
                break;
            }
        case 0x0004:// URL
            {
                if (DEBUG)
                    misc_log("Got URL.");
                
                parse_url(packet, online);
                break;
            }
        case 0x000C:// You've been added.
            {
                if (DEBUG)
                    misc_log("Got You've been added.");
                
                parse_added(packet, online);
                break;
            }
    }
}


/*
** 
** Got a message, translate it, and notify the UI
*/
void parse_message(ICQ_PACKET *packet, int online)
{
    unsigned long int *uin  =  (unsigned long int *)(packet->data + 6);

    if (online) {
        unsigned short int *msgLength = (unsigned short int *)(packet->data + 12);
        unsigned char *msg =            (unsigned char *)(packet->data + 14);

        time_t t = time(NULL);
        tm *t2   = localtime(&t);

        MESSAGE message;
        message.online     = 1;
        message.time.year  = t2->tm_year + 1900;
        message.time.month = t2->tm_mon + 1;
        message.time.day   = t2->tm_mday;
        message.time.hour  = t2->tm_hour;
        message.time.min   = t2->tm_min;
        message.sender     = *uin;
        message.length     = *msgLength;
        message.message    = (char *)msg;

        (*callback.message)(&message);
    } else {
        unsigned short int *year =      (unsigned short int *)(packet->data + 10);
        unsigned char *month =          (unsigned char *)(packet->data + 12);
        unsigned char *day =            (unsigned char *)(packet->data + 13);
        unsigned char *hour =           (unsigned char *)(packet->data + 14);
        unsigned char *min =            (unsigned char *)(packet->data + 15);
        unsigned short int *msgLength = (unsigned short int *)(packet->data + 18);
        unsigned char *msg =            (unsigned char *)(packet->data + 20);

        MESSAGE message;
        message.online     = 0;
        message.time.year  = *year;
        message.time.month = *month;
        message.time.day   = *day;
        message.time.hour  = *hour;
        message.time.min   = *min;
        message.sender     = *uin;
        message.length     = *msgLength;
        message.message    = (char *)msg;

        (*callback.message)(&message);
    }
}


/*
** 
** Got an URL, translate it, and notify the UI
*/
void parse_url(ICQ_PACKET *packet, int online)
{
    unsigned long int *uin  =  (unsigned long int *)(packet->data + 6);

    if (online)
    {
        unsigned char *msg =            (unsigned char *)(packet->data + 14);

        char *string;
        string = (char *)msg;
        //string = strdup((char *)msg);
        char tmp[2];
        tmp[0] = 0xFE;
        tmp[1] = '\0';
        char *desc = strsep(&string, tmp);
        char *_url  = strsep(&string, "");

        time_t t = time(NULL);
        tm *t2   = localtime(&t);
        
        URL url;
        url.online     = 1;
        url.time.year  = t2->tm_year + 1900;
        url.time.month = t2->tm_mon + 1;
        url.time.day   = t2->tm_mday;
        url.time.hour  = t2->tm_hour;
        url.time.min   = t2->tm_min;
        url.sender = *uin;
        url.url    = (char *)_url;
        url.desc   = (char *)desc;

        (*callback.url)(&url);
    }
    else
    {
        unsigned short int *year =      (unsigned short int *)(packet->data + 10);
        unsigned char *month =          (unsigned char *)(packet->data + 12);
        unsigned char *day =            (unsigned char *)(packet->data + 13);
        unsigned char *hour =           (unsigned char *)(packet->data + 14);
        unsigned char *min =            (unsigned char *)(packet->data + 15);
        unsigned char *msg =            (unsigned char *)(packet->data + 20);

        char *string;
        string = (char *) msg;
        //string = strdup((char *)msg);
        char tmp[2];
        tmp[0] = 0xFE;
        tmp[1] = '\0';
        char *desc = strsep(&string, tmp);
        char *_url  = strsep(&string, "");

        URL url;
        url.time.year  = *year;
        url.time.month = *month;
        url.time.day   = *day;
        url.time.hour  = *hour;
        url.time.min   = *min;
        url.sender     = *uin;
        url.url        = (char *)_url;
        url.desc       = (char *)desc;

        (*callback.url)(&url);
    }
}


/*
** 
** You were added to a contact list, find out whos, and notify the UI
*/
void parse_added(ICQ_PACKET *packet, int online)
{
    unsigned long int *uin  =  (unsigned long int *)(packet->data + 6);

    if (online)
    {
        unsigned char *msg =            (unsigned char *)(packet->data + 14);

        char *string;
        string = (char *)msg;
        //string = strdup((char *)msg);
        char tmp[2];
        tmp[0] = 0xFE;
        tmp[1] = '\0';

        char *nick          = strsep(&string, tmp);
        char *firstName     = strsep(&string, tmp);
        char *lastName      = strsep(&string, tmp);
        char *email         = strsep(&string, tmp);
        char *auth          = strsep(&string, "");

        time_t t = time(NULL);
        tm *t2   = localtime(&t);

        ADDED added;
        added.online     = 1;
        added.time.year  = t2->tm_year;
        added.time.month = t2->tm_mon + 1;
        added.time.day   = t2->tm_mday;
        added.time.hour  = t2->tm_hour;
        added.time.min   = t2->tm_min;
        added.sender     = *uin;
        added.nick       = (char *)nick;
        added.firstName  = (char *)firstName;
        added.lastName   = (char *)lastName;
        added.email      = (char *)email;

        if (auth[0] == '0')
            added.auth = 0;// You need authorization to add this user
        else
            added.auth = 1;// You don't need authorization to add this user
        
        (*callback.added)(&added);
    }
    else
    {
        unsigned short int *year =      (unsigned short int *)(packet->data + 10);
        unsigned char *month =          (unsigned char *)(packet->data + 12);
        unsigned char *day =            (unsigned char *)(packet->data + 13);
        unsigned char *hour =           (unsigned char *)(packet->data + 14);
        unsigned char *min =            (unsigned char *)(packet->data + 15);
        unsigned char *msg =            (unsigned char *)(packet->data + 20);

        char *string;
        string = (char *)msg;
        //string = strdup((char *)msg);
        char tmp[2];
        tmp[0] = 0xFE;
        tmp[1] = '\0';
        char *nick          = strsep(&string, tmp);
        char *firstName     = strsep(&string, tmp);
        char *lastName      = strsep(&string, tmp);
        char *email         = strsep(&string, tmp);
        char *auth          = strsep(&string, "");

        ADDED added;
        added.online     = 0;
        added.time.year  = *year;
        added.time.month = *month;
        added.time.day   = *day;
        added.time.hour  = *hour;
        added.time.min   = *min;
        added.sender     = *uin;
        added.nick       = (char *)nick;
        added.firstName  = (char *)firstName;
        added.lastName   = (char *)lastName;
        added.email      = (char *)email;

        if (auth[0] == '0')
            added.auth = 0;// You need authorization to add the user
        else
            added.auth = 1;// You don't need authorization to add this user

        (*callback.added)(&added);
    }
}


/*
**
** A user was found matching our latest search query
*/
void parse_S_USER_FOUND(ICQ_PACKET *packet)
{
    unsigned long int       *uin;
    unsigned short int      *nickLength;
    unsigned char           *nick;
    unsigned short int      *firstLength;
    unsigned char           *first;
    unsigned short int      *lastLength;
    unsigned char           *last;
    unsigned short int      *emailLength;
    unsigned char           *email;
    unsigned char           *auth;

    int pos = 8;

    uin =         (unsigned long int *)(packet->data + pos);     pos += 4;
    nickLength =  (unsigned short int *)(packet->data + pos);    pos += 2;
    nick =        (unsigned char *)(packet->data + pos);         pos += *nickLength;
    firstLength = (unsigned short int *)(packet->data + pos);    pos += 2;
    first =       (unsigned char *)(packet->data + pos);         pos += *firstLength;
    lastLength =  (unsigned short int *)(packet->data + pos);    pos += 2;
    last =        (unsigned char *)(packet->data + pos);         pos += *lastLength;
    emailLength = (unsigned short int *)(packet->data + pos);    pos += 2;
    email =       (unsigned char *)(packet->data + pos);         pos += *emailLength;
    auth =        (unsigned char *)(packet->data + pos);

    SEARCH_RESULT result;
    result.uin =       *uin;
    result.nick =      (char *)nick;
    result.firstName = (char *)first;
    result.lastName =  (char *)last;
    result.email =     (char *)email;
    if (auth[0] == '0')
        result.auth = 0;// You need authorization to add the user
    else
        result.auth = 1;// You don't need authorization to add this user

    (*callback.searchResult)(&result);
}
