/**************************************************************************
* libXicq - Originally written by tnc (xtrophy@it.dk)                     *
*'''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''*
* structs.h                                                               *
* Structures used in libXicq                                              *
**************************************************************************/


#ifndef _STRUCTS_H
#define _STRUCTS_H


/*
** A little info on the lib, name, version, author etc.
** Please use this when (or should I say if?) you greet me.
*/
struct LIB_INFORMATION
{
    const char NAME[]		= "libXicq";
    const char VERSION[]	= "062398-snapshot";
    const char AUTHOR[]		= "tnc";
    const char CONTACT[]	= "xtrophy@it.dk";
};



/*
**
** Holds all config stuff, like which server to use, uin, pwd etc
*/
struct USER_CONFIGURATION {
    unsigned long int	uin;
    char		*password;
    char		*nick;
    char		*firstName;
    char		*lastName;
    char		*email;
    unsigned long int	status;
    unsigned long int	defaultStatus;
    int			showoffline;
    char		*contactList;
    char		*serverAddr;
    int			serverPort;
    int			clientPort;
    char		*logfile;
};

/*
struct USER_CONFIGURATION
{
    unsigned long  int uin;
    char               password[NAMESIZE];
    char               nick[NAMESIZE];
    char               firstName[NAMESIZE];
    char               lastName[NAMESIZE];
    char               email[BUFSIZE];
    unsigned long  int status;
    unsigned long  int defaultStatus;
    int                showoffline;
    char               contactList[BUFSIZE];
    char               serverAddr[BUFSIZE];
    int                serverPort;
    int                clientPort;
};
*/

/*
**
** Holds all networking information, like addresses, ports etc
*/
struct NET_INFORMATION
{
    char *serverAddr;	// The icq server we're using
    int  serverPort;	// Port to use on the server
    int  sock;

    char *clientAddr;	// Our ip
    int  clientPort;	// Our listen port
};

/*
struct NET_INFORMATION
{
    char serverAddr[BUFSIZE];   // The icq server we're using
    int  serverPort;            // Port to use on the server
    int  sock;

    char clientAddr[BUFSIZE];   // Our ip
    int  clientPort;            // Our listen port
};
*/

/*
**
** Packet structure, length, data, command, sequence etc.
*/
struct ICQ_PACKET
{
    int                length;
    unsigned char      data[BUFSIZE];
    unsigned short int command;
    unsigned short int sequence;
};


/*
**
** Structure of the contact list
*/
struct CONTACT
{
    char               nick[20];
    unsigned long  int uin;
    unsigned long  int status;
    unsigned long  int ip;
    unsigned long  int real_ip;
    unsigned short int port;
};


/*
**
** Time struct
*/
struct TIME
{
    unsigned short int year;
    unsigned char      month;
    unsigned char      day;
    unsigned char      hour;
    unsigned char      min;
};


/*
**
** Struct for messages
*/
struct MESSAGE
{
    int                online; // 1=Online, 0=Offline
    TIME time;                 // Online=Localtime, Offline=Servertime
    unsigned long  int sender;
    int                length;
    char              *message;
};


/*
**
** Struct for URLs
*/
struct URL
{
    int                online;
    TIME time;
    unsigned long  int sender;
    char              *url;
    char              *desc;
};


/*
**
** Struct for "You've been added" msgs
*/
struct ADDED
{
    int                online;
    TIME time;
    unsigned long  int sender;
    char              *nick;
    char              *firstName;
    char              *lastName;
    char              *email;
    int                auth;
};


/*
**
** Struct for Search results
*/
struct SEARCH_RESULT
{
    unsigned long int  uin;
    char              *nick;
    char              *firstName;
    char              *lastName;
    char              *email;
    int                auth;
};


/*
**
** Holds pointers to the callback functions the UI set up
*/
struct CALLBACK_FUNCTIONS
{
    void (*loggedIn)(void); // We're logged in
    void (*disconnected)(void);// We lost the connection
    void (*updateContact)(unsigned long int uin); // Contact list was updated
    void (*message)(MESSAGE *message);// Got a message
    void (*url)(URL *url);// Got an URL
    void (*added)(ADDED *added);// Was added to contact list
    void (*searchResult)(SEARCH_RESULT *result);// Result of search query
};


#endif
