/**************************************************************************
* Xtrophy ICQ - Originally written by tnc (xtrophy@it.dk)                 *
*'''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''*
* callbacks.cpp                                                           *
* Functions called by libXicq                                             *
**************************************************************************/


#include "callbacks.h"
#include "io.h"



/*
**
** Setup the callback functions for the lib
*/
void callbackSetup(CALLBACK_FUNCTIONS &callback_functions)
{
    callback_functions.loggedIn      = loggedIn;
    callback_functions.disconnected  = disconnected;
    callback_functions.updateContact = updateContact;
    callback_functions.message       = message;
    callback_functions.url           = url;
    callback_functions.added         = added;
    callback_functions.searchResult  = searchResult;
}


void loggedIn()
{
    extern int logged_in;
    logged_in = 1;

    MY_PRINTF(color.none, "\n");
    makePrompt();
}


void disconnected()
{
    MY_PRINTF(color.none, "\n\nDisconnected!\n");
    
    extern int logged_in;
    logged_in = -1;
}


void updateContact(unsigned long int uin)
{
    CONTACT contact;
    time_t timestamp;
    struct tm *ts;
    timestamp = time(NULL);
    ts = localtime(&timestamp);
    
    contact = contact_returnContact(uin);
    MY_PRINTF(color.text, "\n[%02d/%02d/%02d - %02d:%02d] %s is %s.\n", 
              ts->tm_year, ts->tm_mon + 1, ts->tm_mday, ts->tm_hour, ts->tm_min,
              contact.nick, contact_status2str(contact.status));
    MY_LOGF(color.text, "", "[%02d/%02d/%02d - %02d:%02d] %s is %s.\n", 
            ts->tm_year, ts->tm_mon + 1, ts->tm_mday, ts->tm_hour, ts->tm_min,
            contact.nick, contact_status2str(contact.status));
    make_prompt();
}


void message(MESSAGE *message)
{
    MY_PRINTF(color.none, "\a");
    MY_PRINTF(color.border, "\n===============================================================================\n");
    if (message->online)
	MY_LAPRINTF(color.header, contact_uin2nick(message->sender), 
		 "Message from %s on %02d/%02d/%d at %02d:%02d\n", 
		 contact_uin2nick(message->sender), message->time.month, 
		 message->time.day, message->time.year, message->time.hour, 
		 message->time.min);
    else
	MY_LAPRINTF(color.header, contact_uin2nick(message->sender), 
	    "Message from %s sent while offline on %02d/%02d/%d at %02d:%02d\n",
	    contact_uin2nick(message->sender), message->time.month, 
	    message->time.day, message->time.year, message->time.hour, 
	    message->time.min);
    MY_PRINTF(color.border, "-------------------------------------------------------------------------------\n");
    MY_LAPRINTF(color.text, contact_uin2nick(message->sender),   "%s\n", message->message);
    MY_LOGF(color.text, contact_uin2nick(message->sender), "-----------\n");
    MY_PRINTF(color.border, "===============================================================================\n");

    makePrompt();
}


void url(URL *url)
{
    MY_PRINTF(color.none, "\a");
    MY_LAPRINTF(color.border, contact_uin2nick(url->sender), "\n===============================================================================\n");
    if (url->online)
	MY_LAPRINTF(color.header, contact_uin2nick(url->sender), "URL from %s on %02d/%02d/%d at %02d:%02d\n", contact_uin2nick(url->sender), url->time.month, url->time.day, url->time.year, url->time.hour, url->time.min);
    else
	MY_LAPRINTF(color.header, contact_uin2nick(url->sender), "URL from %s sent while offline on %02d/%02d/%d at %02d:%02d\n", contact_uin2nick(url->sender), url->time.month, url->time.day, url->time.year, url->time.hour, url->time.min);
    MY_LAPRINTF(color.border, contact_uin2nick(url->sender), "-------------------------------------------------------------------------------\n");
    MY_LAPRINTF(color.text, contact_uin2nick(url->sender),   "Description: %s\n", url->desc);
    MY_LAPRINTF(color.text, contact_uin2nick(url->sender),   "URL        : %s\n", url->url);
    MY_LAPRINTF(color.border, contact_uin2nick(url->sender), "===============================================================================\n");

    makePrompt();
}


void added(ADDED *added)
{
    MY_PRINTF(color.border, "\a\n===============================================================================\n");
    if (added->online)
        MY_PRINTF(color.header, "%s added you to his/her contact list:\n", added->nick);
    else
        MY_PRINTF(color.header, "%s added you to his her contact list on %02d/%02d/%04d at %02d:%02d:\n", added->nick, added->time.month, added->time.day, added->time.year, added->time.hour, added->time.min);
    MY_PRINTF(color.border, "-------------------------------------------------------------------------------\n");
    MY_PRINTF(color.text,   "UIN  : %ld\n", added->sender);
    MY_PRINTF(color.text,   "first: %s\n", added->firstName);
    MY_PRINTF(color.text,   "last : %s\n", added->lastName);
    MY_PRINTF(color.text,   "email: %s\n", added->email);
    MY_PRINTF(color.text,   "auth : %s\n", added->auth == 0 ? "Yes" : "No");
    MY_PRINTF(color.border, "===============================================================================\n");

    makePrompt();
}

void searchResult(SEARCH_RESULT *result)
{
    MY_PRINTF(color.border, "\a\n===============================================================================\n");
    MY_PRINTF(color.header, "Info on %s:\n", result->nick);
    MY_PRINTF(color.border, "-------------------------------------------------------------------------------\n");
    MY_PRINTF(color.text,   "UIN  : %ld\n", result->uin);
    MY_PRINTF(color.text,   "first: %s\n", result->firstName);
    MY_PRINTF(color.text,   "last : %s\n", result->lastName);
    MY_PRINTF(color.text,   "email: %s\n", result->email);
    MY_PRINTF(color.text,   "auth : %s\n", result->auth == 0 ? "Yes" : "No");
    MY_PRINTF(color.border, "===============================================================================\n");

    makePrompt();
}

