/**************************************************************************
* Xtrophy ICQ - Originally written by tnc (xtrophy@it.dk)                 *
*'''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''''*
* cmd_parse.cc                                                            *
* Parse keyboard input                                                    *
**************************************************************************/


#include "cmd_parse.h"
#include "io.h"


extern USER_CONFIGURATION user_cfg;
extern void makePrompt();
int (*aux_input_handler)(char *input) = NULL;


MSG_STRUCT msg;
URL_STRUCT _url;
int search_sequence = 0;


int cmd_parse(char *input)
{
    if (aux_input_handler != NULL)
	{
	    int i;
	    i = aux_input_handler(input);
	    makePrompt();
	    return i;
	}

    char *tmp;
    char *tmp2;
    tmp = strdup(input);
    tmp2 = tmp;
    char *cmd;
    cmd = strsep(&tmp, " ");

    if (cmd == NULL);
    else if (*cmd == '\0');
    else if (!strcasecmp(cmd, "help")) cmd_help(input);
    else if (!strcasecmp(cmd, "quit")) return 0;
    else if (!strcasecmp(cmd, "exit")) return 0;
    else if (!strcasecmp(cmd, "msg")) cmd_msg(input);
    else if (!strcasecmp(cmd, "url")) cmd_url(input);
    else if (!strcasecmp(cmd, "who")) cmd_who(input);
    else if (!strcasecmp(cmd, "status")) cmd_status(input);
    else if (!strcasecmp(cmd, "clear")) cmd_clear(input);
    else if (!strcasecmp(cmd, "info")) cmd_search_uin(input);
    else if (!strcasecmp(cmd, "talk")) cmd_talk(input);
    else MY_PRINTF(color.none, "Huh?!? You need 'help'?\n");

    free((void *) tmp2);

    makePrompt();

    return 1;
}// cmd_parse


int cmd_help(char *input)
{
    if (!strcasecmp(input, "help"))
    {
        MY_PRINTF(color.border, "===============================================================================\n");
        MY_PRINTF(color.header, "Command index:\n");
        MY_PRINTF(color.border, "-------------------------------------------------------------------------------\n");
        MY_PRINTF(color.text,   "clear                                  info <nick|uin>\n");
        MY_PRINTF(color.text,   "msg <nick|uin [<msg>]                  quit\n");
        MY_PRINTF(color.text,   "status <stat>                          talk <nick|uin>\n");
        MY_PRINTF(color.text,   "url <nick|uin>                         who\n");
        MY_PRINTF(color.text,   "\n");
        MY_PRINTF(color.text,   "help <command> to get a more detailed description.\n");
        MY_PRINTF(color.border, "===============================================================================\n");
    }
    else
        help(input);

    return 0;
}


int cmd_msg(char *input)
{
    if (strlen(input) > 3)
    {
	char *tmp;
	tmp = strtok(input, " ");
	tmp = strtok(NULL, " ");
	if (tmp)
	{
	    unsigned long int uin;
	    uin = contact_nick2uin(tmp);
	    if (uin)
	    {
		tmp = strtok(NULL, "");
		if (tmp)
		{
		    time_t timestamp;
		    struct tm *ts;
		    timestamp = time(NULL);
		    ts = localtime(&timestamp);

		    send_message(uin, tmp);
		    MY_LOGF(color.text, contact_uin2nick(uin), 
			    "[%02d/%02d/%02d - %02d:%02d]\n",
			    ts->tm_year, ts->tm_mon + 1, 
			    ts->tm_mday, ts->tm_hour, ts->tm_min);
		    MY_LOGF(color.text, contact_uin2nick(uin), "%s\n", tmp);
		    MY_LOGF(color.text, contact_uin2nick(uin), "-----------\n");
		    MY_PRINTF(color.none, "Sending message to %s.\n", contact_uin2nick(uin));

		    return 0;
		}
		else
		{
		    if (aux_input_handler != NULL || aux_prompt != NULL) {
			// This REALLY should /NOT/ happen, yell, loudly..
			MY_PRINTF(color.none, "\nAieeeeeeee!! Something is really fscked, bailing out..\n");
			my_exit(1);
		    }
		    aux_input_handler = msg_input_handler;
		    aux_prompt = strdup("> ");
		    MY_PRINTF(color.none, "\nEnter message to %s, '.' on a newline to send, '.abort' on a newline to abort.\n", contact_uin2nick(uin));
		    fflush(stdout);
		    msg.messageLength = 0;
		    msg.message[0] = '\0';
		    msg.uin = uin;
	       }
	   }
	   else
	   {
	       MY_PRINTF(color.none, "Uhm.. %s? Who's that?\n", tmp);
	   }
       }
    }
    else
    {
	MY_PRINTF(color.none, "Syntax: msg <nick|uin> [<msg>]\n");
    }

    return -1;
}// cmd_msg

int msg_input_handler(char *msgTmp) {
    int msgTmpLength = 0;

    int msgDone = 0;

    msgTmpLength = strlen(msgTmp);
    if (!(msgTmpLength == 1 && msgTmp[0] == '.'))
	if (msg.messageLength + msgTmpLength < BUFSIZE)
	    strcat(msg.message, msgTmp);
	else {
	    strncat(msg.message, msgTmp, BUFSIZE - msg.messageLength - 2);
	    MY_PRINTF(color.none, "Xicq: Your message it too long, sending it now.\n");
	    msgDone = 1;
	}

    if (msgTmpLength == 1 && msgTmp[0] == '.') {
	msg.message[strlen(msg.message) - 2] = '\0';
	msgDone = 1;
    } else if (!(strcasecmp(msgTmp, ".abort"))) {
	MY_PRINTF(color.none, "%sAborted.\n", aux_prompt);
	msg.message[strlen(msg.message) - 7] = '\0';
	msgDone = 2;
    } else {
	msg.messageLength = strlen(msg.message);
	msg.message[msg.messageLength] = '\r';
	msg.message[msg.messageLength + 1] = '\n';
	msg.message[msg.messageLength + 2] = '\0';
	msg.messageLength++;
    }

    if (msgDone == 1) {
	send_message(msg.uin, msg.message);
	time_t timestamp;
	struct tm *ts;
	timestamp = time(NULL);
	ts = localtime(&timestamp);
    
	MY_LOGF(color.text, contact_uin2nick(msg.uin),
		"[%02d/%02d/%02d - %02d:%02d]\n",
		ts->tm_year, ts->tm_mon + 1,
		ts->tm_mday, ts->tm_hour, ts->tm_min);
	MY_LOGF(color.text, contact_uin2nick(msg.uin), "%s\n", msg.message);
	MY_LOGF(color.text, contact_uin2nick(msg.uin), "-----------\n");
	MY_PRINTF(color.none, "Sending message to %s.\n", contact_uin2nick(msg.uin));
	aux_input_handler = NULL;
	free (aux_prompt);
	aux_prompt = NULL;
    } else if (msgDone == 2) {
	// We were aborted...
	aux_input_handler = NULL;
	free (aux_prompt);
	aux_prompt = NULL;
    }
    return 1;
}


int cmd_url(char *input)
{
    if (strlen(input) > 3)
    {
       char *tmp;
       tmp = strtok(input, " ");
       tmp = strtok(NULL, "");
       if (tmp)
       {
           unsigned long int uin;
           uin = contact_nick2uin(tmp);
           if (uin)
           {
                if (aux_input_handler != NULL || aux_prompt != NULL) {
                    // This REALLY should /NOT/ happen, yell, loudly..
                    MY_PRINTF(color.none, "\nAieeeeeeee!! Something is really fscked, bailing out..\n");
                    my_exit(1);
                }
                aux_input_handler = url_input_handler;
                aux_prompt = strdup("URL > ");
                MY_PRINTF(color.none, "\nEnter URL to %s.\n", contact_uin2nick(uin));
                fflush(stdout);
                _url.urlLength = 0;
                _url.url[0] = '\0';
                _url.descLength = 0;
                _url.desc[0] = '\0';
                _url.uin = uin;
          }
          else
          {
              MY_PRINTF(color.none, "Uhm.. %s? Who's that?\n", tmp);
          }
       }
    }
    else
    {
       MY_PRINTF(color.none, "Syntax: url <nick|uin>\n");
    }

    return -1;
}// cmd_url


int url_input_handler(char *urlTmp)
{
    int urlTmpLength = 0;

    int urlDone = 0;

    if (_url.urlLength == 0)
    {
        _url.urlLength = strlen(urlTmp);
        strcpy(_url.url, urlTmp);

        //printf("\n\n%s\n\n", _url.url);

        free(aux_prompt);
        aux_prompt = strdup("Desc> ");
        //makePrompt();

        return 1;
    }

    urlTmpLength = strlen(urlTmp);
    if (!(urlTmpLength == 1 && urlTmp[0] == '.'))
       if (_url.descLength + _url.urlLength + urlTmpLength < BUFSIZE)
           strcat(_url.desc, urlTmp);
        else
        {
           strncat(_url.desc, urlTmp, BUFSIZE - _url.descLength - 2);
           MY_PRINTF(color.none, "Xicq: Your description it too long, sending it now.\n");
           urlDone = 1;
       }

    if (urlTmpLength == 1 && urlTmp[0] == '.')
    {
       _url.desc[strlen(_url.desc) - 2] = '\0';
       urlDone = 1;
    }
    else if (!(strcasecmp(urlTmp, ".abort")))
    {
        MY_PRINTF(color.none, "%sAborted.\n", aux_prompt);
       _url.desc[strlen(_url.desc) - 7] = '\0';
       urlDone = 2;
    }
    else
    {
       _url.descLength = strlen(_url.desc);
       _url.desc[_url.descLength] = '\r';
       _url.desc[_url.descLength + 1] = '\n';
       _url.desc[_url.descLength + 2] = '\0';
       _url.descLength++;
    }

    if (urlDone == 1)
    {
       send_url(_url.uin, _url.url, _url.desc);
       MY_PRINTF(color.none, "%sSending URL to %s.\n", aux_prompt, contact_uin2nick(_url.uin));
       aux_input_handler = NULL;
       free (aux_prompt);
       aux_prompt = NULL;
    }
    else if (urlDone == 2)
    {
       // We were aborted...
       aux_input_handler = NULL;
       free (aux_prompt);
       aux_prompt = NULL;
    }

    return 1;
}

int cmd_who(char *input)
{
    CONTACT contact;

    if (user_cfg.showoffline)
    {
        MY_PRINTF(color.border, "===============================================================================\n");
        MY_PRINTF(color.header, "Offline:\n");
        MY_PRINTF(color.border, "-------------------------------------------------------------------------------\n");
        for (int i = 0; i < contact_length(); i++)
        {
            contact = contact_returnItemNum(i);

            if (contact.status == 0xFFFFFFFF)
                MY_PRINTF(color.text, "%20s%10ld   %s\n", contact_uin2nick(contact.uin), contact.uin, contact_status2str(contact.status));
        }
    }

    MY_PRINTF(color.border, "===============================================================================\n");
    MY_PRINTF(color.header, "Online:\n");
    MY_PRINTF(color.border, "-------------------------------------------------------------------------------\n");

    for (int i = 0; i < contact_length(); i++)
    {
        contact = contact_returnItemNum(i);

        if (contact.status != 0xFFFFFFFF)
            MY_PRINTF(color.text, "%20s%10ld   %s\n", contact_uin2nick(contact.uin), contact.uin, contact_status2str(contact.status));
    }

    MY_PRINTF(color.border, "===============================================================================\n");

    return 0;
}// cmd_who


int cmd_status(char *input)
{
    if (strlen(input) > 6)
    {
	char *tmp;
	tmp = strtok(input, " ");
	tmp = strtok(NULL, "");
        if (tmp)
        {
	    int status;
	    status = status_str2num(tmp);
	    //status = atoi(tmp);
            if (status == -1)
            {
		MY_PRINTF(color.none, "Unknown status.\n");
            }
            else
            {
		send_statusChange(status);
		MY_PRINTF(color.none, "Ok.\n");

		return 0;
	    }
        }
        else
        {
	    MY_PRINTF(color.none, "Syntax: status <stat>\n");
	    MY_PRINTF(color.none, "  stat: 0 = Available              1 = Away\n");
	    MY_PRINTF(color.none, "        2 = DND                    3 = N/A\n");
	    MY_PRINTF(color.none, "        4 = Occupied               5 = Free For Chat\n");
	    MY_PRINTF(color.none, "        6 = Invisible\n");
	    MY_PRINTF(color.none, "Your current status is: %s.\n", returnMyStatus());
        }
    }
    else
    {
        MY_PRINTF(color.none, "Syntax: status <stat>\n");
        MY_PRINTF(color.none, "  stat: 0 = Available              1 = Away\n");
        MY_PRINTF(color.none, "        2 = DND                    3 = N/A\n");
        MY_PRINTF(color.none, "        4 = Occupied               5 = Free For Chat\n");
        MY_PRINTF(color.none, "        6 = Invisible\n");
        MY_PRINTF(color.none, "Your current status is: %s.\n", returnMyStatus());
    }

    return -1;
}// cmd_status


int cmd_search_uin(char *input)
{
    if (strlen(input) > strlen("info "))
    {
        char *tmp;
        tmp = strtok(input, " ");
        tmp = strtok(NULL, "");

        if (tmp)
        {
            unsigned long int uin;
            //uin = (unsigned long int)atoi(tmp);
            uin = contact_nick2uin(tmp);

            send_uinSearch(uin);
            MY_PRINTF(color.none, "Ok.\n");
        }
        else
            MY_PRINTF(color.none, "Syntax: info <nick|uin>\n");
    }
    else
        MY_PRINTF(color.none, "Syntax: info <nick|uin>\n");

    return 0;
}


int cmd_talk(char *input)
{
    if (strlen(input) > strlen("talk "))
    {
       char *tmp;
       tmp = strtok(input, " ");
       tmp = strtok(NULL, "");
       if (tmp)
       {
           unsigned long int uin;
           uin = contact_nick2uin(tmp);
           if (uin)
            {
                char strTmp[BUFSIZE];
                sprintf(strTmp, "%s> ", contact_uin2nick(uin));
                aux_prompt = strdup(strTmp);
                msg.uin = uin;
                msg.messageLength = 0;
                msg.message[0] = '\0';

                aux_input_handler = talk_input_handler;

                MY_PRINTF(color.none, "Entering talk session with %s.\n", contact_uin2nick(msg.uin));
                MY_PRINTF(color.none, "'.' to send a message, '.abort' to abort a message, '.exit' to exit session.\n");

                return 0;
            }
            else
                MY_PRINTF(color.none, "Uhm.. %s? Who's that?\n", tmp);
        }
        else
            MY_PRINTF(color.none, "Syntax: talk <nick|uin>\n");
    }
    else
        MY_PRINTF(color.none, "Syntax: talk <nick|uin>\n");

    return -1;
}// cmd_talk



int talk_input_handler(char *msgTmp)
{
    if (!strcasecmp(msgTmp, ".exit"))
    {
        aux_input_handler = NULL;
        free(aux_prompt);
        aux_prompt = NULL;

        MY_PRINTF(color.none, "Exited talk session.\n");

        //makePrompt();
    }
    else
    {
        msg_input_handler(msgTmp);

        if (aux_input_handler == NULL)
        {
            char strTmp[BUFSIZE];
            MY_PRINTF(color.none, "\n");
            sprintf(strTmp, "%s> ", contact_uin2nick(msg.uin));
            aux_prompt = strdup(strTmp);
            msg.messageLength = 0;
            msg.message[0] = '\0';

            aux_input_handler = talk_input_handler;
        }
    }

    return 1;
}// talk_input_handler

